﻿using System;
using RestSharp;
using System.Net;
using com.alu.ot.ics.api.samples.model;
using RestSharp.Authenticators;

namespace com.alu.ot.ics.api.samples
{
    /// <summary>
    /// This is a sample code of basic usage of the OpenTouch 2.0 REST API.
    /// The project build under VS2010. 
    /// The only external dependencies are the RestSharp library (see http://restsharp.org/),
    /// and the commandLineParser (https://commandline.codeplex.com)
    /// 
    /// The detailed steps are the following:
    /// <ul>
    /// <li>Get the API entry point, giving the authentication URL.</li>
    /// <li>Use this authentication URL to authentication the user on the system.</li>
    /// <li>Open a  session with the given credential.</li>
    /// <li>Make a call.</li>
    /// <li>Close the session.</li>
    /// </ul>
    /// </summary>
    public class BasicOpenTouchRestClient
    {
        /// <summary>
        /// Server host ip address (or hostname if FQDN can resolved by DNS).
        /// </summary>
        public String host {get; private set; }

        /// <summary>
        /// The user's login.
        /// </summary>
        public String login { get; private set; }
        
        /// <summary>
        /// The user's password.
        /// </summary>
        public String password { get; private set; }

        /// <summary>
        /// The user's device to be used for outgoing calls.
        /// </summary>
        public String currentDevice { get; private set; }

        /// <summary>
        /// Contains the user's credential, and the URL to use for opening a session.
        /// </summary>
        public Authentication authenticationInformation { get; private set; }

        /// <summary>
        /// Object returned when invoking an 'open session' request.
        /// Contains usefull information about supported services, capabilities, ... for the user.
        /// </summary>
        public SessionInfo sessionInformation { get; private set; }

        
        /// <summary>
        /// Basic client constructor.
        /// </summary>
        public BasicOpenTouchRestClient()
        {
                                  
        }


        /// <summary>
        /// Acces to the REST API entry point. Provides supported versions and authentication URLs.
        /// Then authenticate using the provided URL.
        /// </summary>
        /// <param name="pHost"></param>
        /// <param name="pLogin"></param>
        /// <param name="pPwd"></param>
        public bool connect(String pHost, String pLogin, String pPwd) 
        {
            bool isItOK = false;
            host = pHost;
            login = pLogin;
            password = pPwd;

            //Use the RestSharp library as a light and yet powerful REST client.
            var rest = new RestClient("http://" + host + "/api/rest");
            rest.UserAgent = "RESTAPI-TestTool";
            RestRequest request = new RestRequest("", Method.GET);
            rest.Timeout = 10000;

            // Send the request and deserialise it in our 'business model objects'.
            IRestResponse<OTRESTVersions> response = rest.Execute<OTRESTVersions>(request);

            //Test the response HTTP code for errors, 
            if (response.StatusCode == HttpStatusCode.OK)
            {
                OTRESTVersion version = getSelectedVersion(response.Data, "1.0");
                authenticationInformation = authenticate(version.internalUrl);
                isItOK = true;
            }
            else
            {
                Console.WriteLine("Error:" + response.StatusCode + " / " + response.ResponseStatus);
            }
            return isItOK;
        }

        /// <summary>
        /// Authentication. 
        /// The client application must follow HTTP redirection, and accept cookies,
        /// since the credential will be stored in a dedicated cookie.
        /// </summary>
        /// <param name="authUrl"></param>
        /// <returns></returns>
        private Authentication authenticate(String authUrl)
        {
            int nbTry = 1;

            var rest = new RestClient(authUrl);
            rest.Authenticator = new HttpBasicAuthenticator(login, password);
            RestRequest requestAuthInfo = new RestRequest("", Method.GET);

            IRestResponse<Authentication> responseAuthInfo = rest.Execute<Authentication>(requestAuthInfo);

            while (nbTry < 5 && responseAuthInfo.StatusCode != HttpStatusCode.OK)
            {
                switch (responseAuthInfo.StatusCode)
                {
                    case HttpStatusCode.Redirect:
                        break;

                    case HttpStatusCode.Unauthorized:
                        rest = new RestClient(responseAuthInfo.ResponseUri.AbsoluteUri);
                        rest.Authenticator = new HttpBasicAuthenticator(login, password);
                        requestAuthInfo = new RestRequest("", Method.GET);

                        responseAuthInfo = rest.Execute<Authentication>(requestAuthInfo);
                        break;

                    default:
                        break;
                }
                nbTry++;
            }

            if (responseAuthInfo.StatusCode == HttpStatusCode.OK)
            {
                return responseAuthInfo.Data;
            }
            else
            {
                return null;
            }
        }

        /// <summary>Open a session on the OpenTouch server.</summary>
        /// <param name="pAppName">Token used to identify the application.</param>
        /// <returns></returns>
        public HttpStatusCode openSession(String pAppName)
        {
            var rest = new RestClient(authenticationInformation.internalUrl);

            RestRequest request = new RestRequest("", Method.POST);

            //The AlcUserId is used to store the credential of the user. It must be present in all requests.
            request.AddCookie("AlcUserId", authenticationInformation.credential);
            request.RequestFormat = DataFormat.Json;
            SessionRequest app = new SessionRequest();
            app.applicationName = pAppName;
            request.AddBody(app);

            IRestResponse<SessionInfo> response = rest.Execute<SessionInfo>(request);
            sessionInformation = response.Data;

            return response.StatusCode;
        }


        /// <summary>
        /// Make a call from the given device to the user identified by its compay phone numer
        /// </summary>
        /// <param name="pCalleeNumber"></param>
        /// <param name="pCallerDevice"></param>
        public HttpStatusCode makeCall(String pCallerDevice, String pCalleeNumber)
        {
            var rest = new RestClient(sessionInformation.privateBaseUrl + "/telephony/basicCall");
            RestRequest request = new RestRequest("", Method.POST);
            request.AddCookie("AlcUserId", authenticationInformation.credential);
            request.RequestFormat = DataFormat.Json;

            MakeBasicCallRequest makeCallRequest = new MakeBasicCallRequest();
            makeCallRequest.autoAnswer = true;
            makeCallRequest.callee = pCalleeNumber;
            makeCallRequest.deviceId = pCallerDevice;
            request.AddBody(makeCallRequest);

            IRestResponse response = rest.Execute(request);

            return response.StatusCode;
        }

        /// <summary>
        /// Take the active call on the given device.
        /// </summary>
        /// <param name="pDevice"></param>
        public HttpStatusCode takeCall(String pDevice)
        {
            var rest = new RestClient(sessionInformation.privateBaseUrl + "/telephony/basicCall/answer");
            RestRequest request = new RestRequest("", Method.POST);
            request.AddCookie("AlcUserId", authenticationInformation.credential);
            request.RequestFormat = DataFormat.Json;

            AnswerRequest answerRequest = new AnswerRequest();
            answerRequest.deviceId = pDevice;
            request.AddBody(answerRequest);

            IRestResponse response = rest.Execute(request);

            return response.StatusCode;
        }

        /// <summary>
        /// Drop the active call.
        /// </summary>
        public HttpStatusCode dropCall()
        {
            var rest = new RestClient(sessionInformation.privateBaseUrl + "/telephony/basicCall/dropme");
            RestRequest request = new RestRequest("", Method.POST);
            request.AddCookie("AlcUserId", authenticationInformation.credential);
            request.RequestFormat = DataFormat.Json;

            IRestResponse response = rest.Execute(request);

            return response.StatusCode;
        }

        /// <summary>close the session.</summary>
        public void closeSession()
        {
            var rest = new RestClient(authenticationInformation.internalUrl);
            RestRequest request = new RestRequest("", Method.DELETE);
            request.AddCookie("AlcUserId", authenticationInformation.credential);
            request.RequestFormat = DataFormat.Json;

            IRestResponse response = rest.Execute(request);
        }


        /// <summary>
        /// Get the requested version from the list of supported versions.
        /// </summary>
        /// <returns></returns>
        public OTRESTVersion getSelectedVersion(OTRESTVersions pVersions, String pSelected)
        {
            OTRESTVersion selected = null;
            foreach (OTRESTVersion version in pVersions.versions)
            {
                if (version.id == pSelected)
                {
                    selected = version;
                    break;
                }
            }
            return selected;
        }
    }
}
